@@ -1,6 +1,6 @@
 package Netscape::Bookmarks::Category;
-# $Revision: 1.1.1.1 $
-# $Id: Category.pm,v 1.1.1.1 2002/01/08 16:43:24 comdog Exp $
+# $Revision: 1.4 $
+# $Id: Category.pm,v 1.4 2004/09/16 01:48:30 comdog Exp $
 
 =head1 NAME
 
@@ -9,13 +9,13 @@ Netscape::Bookmarks::Category	- manipulate, or create Netscape Bookmarks files
 =head1 SYNOPSIS
 
   use Netscape::Bookmarks;
-  
+
   #parse an existing file
   my $bookmarks = new Netscape::Bookmarks $bookmarks_file;
-  
+
   #print a Netscape compatible file
   print $bookmarks->as_string;
-  
+
 =head1 DESCRIPTION
 
 The Netscape bookmarks file has several basic components:
@@ -25,7 +25,7 @@ The Netscape bookmarks file has several basic components:
 	links
 	aliases
 	separators
-	
+
 On disk, Netscape browsers store this information in HTML. In the browser,
 it is displayed under the "Bookmarks" menu.  The data can be manipulated
 through the browser interface.
@@ -42,11 +42,11 @@ only argument to the constructor:
 	my $bookmarks = new Netscape::Bookmarks $bookmarks_file;
 
 The returned object is a Netscape::Bookmarks::Category object, since the bookmark file is
-simply a collection of categories that contain any of the components listed 
+simply a collection of categories that contain any of the components listed
 above.  The top level (i.e. root) category is treated specially and defines the
 title of the bookmarks file.
 
-=head1 METHODS
+=head2 Methods
 
 =over 4
 
@@ -56,8 +56,6 @@ use strict;
 use subs qw();
 use vars qw($VERSION $ERROR @EXPORT @EXPORT_OK @ISA $LAST_ID %IDS);
 
-use Netscape::Bookmarks;
-
 use Exporter;
 
 use URI::URL;
@@ -68,8 +66,8 @@ use constant START_LIST_ITEM => '<DT>';
 use constant TAB             => '    ';
 use constant FOLDED_TRUE     => 1;
 use constant FOLDED_FALSE    => 0;
-	
-($VERSION) = q$Revision: 1.1.1.1 $ =~ m/(\d+\.\d+)\d*$/;
+
+($VERSION) = q$Revision: 1.4 $ =~ m/(\d+\.\d+)\d*$/;
 %IDS     = ();
 $LAST_ID = -1;
 
@@ -94,10 +92,10 @@ sub new
 	{
 	my $class  = shift;
 	my $param  = shift;
-	
+
 	my $self = {};
 	bless $self, $class;
-	
+
 	$param->{'folded'} = FOLDED_TRUE unless $param->{'folded'} == FOLDED_FALSE;
 
 	unless( exists $IDS{$param->{'id'}} or $param->{'id'} =~ /\D/)
@@ -106,21 +104,23 @@ sub new
 		$IDS{$LAST_ID}++;
 		}
 
-	if( defined $param->{'add_date'} and $param->{'add_date'} =~ /\D/  )
+	$param->{'add_date'} ||= 0; # get rid of uninit warnings
+	
+	if( $param->{'add_date'} =~ /\D/ or not $param->{'add_date'} =~ /^\d+$/ )
 		{
 		$param->{'add_date'} = 0;
 		}
-	
+
 	$self->{'title'}       = $param->{'title'};
 	$self->{'folded'}      = $param->{'folded'};
 	$self->{'add_date'}    = $param->{'add_date'};
 	$self->{'id'}          = $param->{'id'};
 	$self->{'description'} = $param->{'description'};
 	$self->{'thingys'}     = [];
-	
+
 	$self;
 	}
-	
+
 =item $category-E<gt>add( $object )
 
 The add() function adds an element to a category.  The element must be a Alias,
@@ -130,51 +130,19 @@ Link, Category, or Separator object.  Returns TRUE or FALSE.
 
 sub add
 	{
-	my $self   = shift;
+	my $self = shift;
 	my $thingy = shift;
-	
-	return unless 
-		ref $thingy eq 'Netscape::Bookmarks::Link' or 
-		ref $thingy eq 'Netscape::Bookmarks::Category' or
-		ref $thingy eq 'Netscape::Bookmarks::Separator' or
-		ref $thingy eq 'Netscape::Bookmarks::Alias';
-		
-	push @{ $self->{'thingys'} }, $thingy;
+
+	return unless
+		ref $$thingy eq 'Netscape::Bookmarks::Link' or
+		ref $$thingy eq 'Netscape::Bookmarks::Category' or
+		ref $$thingy eq 'Netscape::Bookmarks::Separator' or
+		ref $$thingy eq 'Netscape::Bookmarks::Alias';
+
+	push @{ $self->{'thingys'} }, $$thingy;
 	}
 
-=item $category-E<gt>remove_element( $object )
-
-Removes the given object from the Category by calling the object's
-remove() method.
-
-Returns the number of objects removed from the Category.
-
-=cut
-
-sub remove_element
-	{
-	my $self   = shift;
-	my $thingy = shift;
-	
-	my $old_count = $self->count;
-	 
-	$self->{'thingys'} = 
-		[ grep { $_ ne $thingy and $_->remove } $self->elements ];
-		
-	return $old_count - $self->count;
-	}
-		
-=item $category-E<gt>remove()
-
-Performs any clean up necessary to remove this object from the
-Bookmarks tree. Although this method does not recursively remove
-objects which it contains, it probably should.
-
-=cut
-
-sub remove { 1; }
-
-# append_title is used by the parser routines to add to a 
+# append_title is used by the parser routines to add to a
 # title as the HTML stream is parsed.  the title may fall
 # across a chunk boundary so the first part is saved and
 # the second part is added.
@@ -182,11 +150,11 @@ sub append_title
 	{
 	my $self = shift;
 	my $text = shift;
-	
+
 	$self->{'title'} .= $text;
 	}
 
-# append_description is used by the parser routines to add to a 
+# append_description is used by the parser routines to add to a
 # title as the HTML stream is parsed.  the title may fall
 # across a chunk boundary so the first part is saved and
 # the second part is added.
@@ -194,10 +162,10 @@ sub append_description
 	{
 	my $self = shift;
 	my $text = shift;
-	
+
 	$self->{'description'} .= $text;
 	}
-	
+
 =item $category-E<gt>add_desc( $object )
 
 Adds a description to the category.
@@ -208,7 +176,7 @@ sub add_desc
 	{
 	my $self = shift;
 	my $text = shift;
-			
+
 	$self->{'description'} = $text;
 	}
 
@@ -221,7 +189,7 @@ Returns title to the category.
 sub title
 	{
 	my $self = shift;
-	
+
 	$self->{'title'};
 	}
 
@@ -234,7 +202,7 @@ Returns the ID of the category. This is an arbitrary, unique number.
 sub id
 	{
 	my $self = shift;
-	
+
 	$self->{'id'};
 	}
 
@@ -261,7 +229,7 @@ Returns the folded state of the category (TRUE or FALSE).  If the category is
 sub folded
 	{
 	my $self = shift;
-	
+
 	return $self->{'folded'} ? 1 : 0;
 	}
 
@@ -274,34 +242,23 @@ Returns the ADD_DATE attribute of the category.
 sub add_date
 	{
 	my $self = shift;
-	
+
 	return $self->{'add_date'};
 	}
-	
+
 =item $category-E<gt>elements()
 
-In scalar context returns an array reference to the elements in 
-the category.  In list context returns a list of the elements in
-the category.
+Returns an array reference to the elements in the category.
 
 =cut
 
 sub elements
 	{
 	my $self = shift;
-	
-	if( wantarray ) { @{ $self->{'thingys'} } }
-	else            {    $self->{'thingys'}   }
+
+	return \@{ $self->{'thingys'} };
 	}
 
-=item $category-E<gt>count()
-
-Returns a count of the number of objects in the Category.
-
-=cut
-
-sub count { scalar @{ $_[0]->{'thingys'} } }
-	
 =item $category-E<gt>categories()
 
 Returns a list of the Category objects in the category.
@@ -311,10 +268,9 @@ Returns a list of the Category objects in the category.
 sub categories
 	{
 	my $self = shift;
-	
-	my @list = grep ref $_ eq 'Netscape::Bookmarks::Category', 
-		$self->elements;
-	
+
+	my @list = grep ref $_ eq 'Netscape::Bookmarks::Category', @{$self->elements};
+
 	return @list;
 	}
 
@@ -327,10 +283,9 @@ Returns a list of the Link objects in the category.
 sub links
 	{
 	my $self = shift;
-	
-	my @list = grep ref $_ eq 'Netscape::Bookmarks::Link',
-		 $self->elements;
-	
+
+	my @list = grep ref $_ eq 'Netscape::Bookmarks::Link', @{$self->elements};
+
 	return @list;
 	}
 
@@ -344,144 +299,49 @@ the elements of the category.
 sub as_headline
 	{
 	my $self = shift;
-	
-	my $folded   = $self->folded ? "FOLDED" : "";
-	my $title    = $self->title;
-	my $desc     = $self->description;
+
+	my $folded = $self->folded ? "FOLDED" : "";
 	my $add_date = $self->add_date;
-		
-	$desc = $desc ne '' ? "\n<DD>$desc" : '';
+	my $title = $self->title;
+	my $desc     = $self->description;
+
+	$desc = "\n<DD>$desc" if $desc ne '';
 
 	$add_date = $add_date ? qq|ADD_DATE="$add_date"| : '';
-	
+
 	my $sp = ($folded and $add_date) ? ' ' : '';
-	
+
 	return qq|<H3 $folded$sp$add_date>$title</H3>$desc|
 	}
 
-=head2 $category-E<gt>recurse( CODE, [ LEVEL ] )
-
-This method performs a depth-first traversal of the Bookmarks
-tree and executes the CODE reference at each node.  
-
-The CODE reference receives two arguments - the object on which
-it should operate and its level in the tree.
-
-=cut
-
-sub recurse
-	{
-	my $self  = shift;
-	my $sub   = shift;
-	my $level = shift || 0;
-	
-	unless( ref $sub eq 'CODE' )
-		{
-		warn "Argument to recurse is not a code reference";
-		return;
-		}
-
-	$sub->( $self, $level );
-		
-	++$level;
-	foreach my $element ( $self->elements )
-		{
-		if( $element->isa( __PACKAGE__ ) )
-			{
-			$element->recurse( $sub, $level );
-			}
-		else
-			{
-			$sub->( $element, $level );
-			}
-		}
-	--$level;
-
-	}	
-	
-=item $category-E<gt>sort_elements( [ CODE ] )
-
-Sorts the elements in the category using the provided CODE
-reference.  If you do not specify a CODE reference, the
-elements are sorted by title (with the side effect of 
-removing Separators from the Category).
-
-This function does not recurse, although you can use
-the recurse() method to do that.
-
-Since the built-in sort() uses the package variables
-C<$a> and C<$b>, your sort subroutine has to make sure
-that it is accessing the right C<$a> and C<$b>, which 
-are the ones in the package C<Netscape::Bookmarks::Category>.
-You can start your CODE reference with a package
-declaration to ensure the right thing happens:
-
-	my $sub = sub {
-		package Netscape::Bookmarks::Category;
-		
-		$b->title cmp $a->title;
-		};
-		
-	$category->sort_elements( $sub );
-	
-If you know a better way to do this, please let me know. :)
-
-=cut
-
-sub sort_elements
-	{
-	my $self = shift;
-	my $sub  = shift;
-		
-	if( defined $sub and not ref $sub eq 'CODE' )
-		{
-		warn "Second argument to sort_elements is not a CODE reference.";
-		return;
-		}
-	elsif( not defined $sub )
-		{
-		$sub = sub { $a->title cmp $b->title };
-		}
-
-	local *my_sorter = $sub;
-		
-	$self->{'thingys'} = [ sort my_sorter
-		grep { not $_->isa( 'Netscape::Bookmarks::Separator' ) }
-			@{ $self->{'thingys'} } ];
-	}
-	
 =item $category-E<gt>as_string()
 
 Returns an HTML string representation of the category as the
-top level category, along with all of the elements of the 
+top level category, along with all of the elements of the
 category and the Categories that it contains, recursively.
 
 =cut
 
 sub as_string
 	{
-	my $self   = shift;
-		
+	my $self = shift;
+
 	my $title = $self->title;
-	my $desc  = $self->description;
-	
+
 	my $str = <<"HTML";
 <!DOCTYPE NETSCAPE-Bookmark-file-1>
 <!-- This is an automatically generated file.
 It will be read and overwritten.
 Do Not Edit! -->
 <TITLE>$title</TITLE>
-<H1>$title</H1>
-
+<H1>$title</H1>\x0A
 HTML
 
-	$str .= "<DD>" . $self->description;
-	
 	$str .= START_LIST . "\n";
-	
-	foreach my $element ( $self->elements )
+
+	foreach my $ref ( @{$self->elements} )
 		{
-		$str .= $self->_as_string( $element, 1 );
+		$str .= $self->_as_string(\$ref, 1);
 		}
 
 	$str .= END_LIST . "\n";
@@ -493,49 +353,67 @@ HTML
 # do.
 sub _as_string
 	{
-	my $self   = shift;
-	my $obj    = shift;
-	my $level  = shift;
-	
-	my $str; 
-	if( ref $obj eq 'Netscape::Bookmarks::Category' )
+	my $self  = shift;
+	my $obj   = shift;
+	my $level = shift;
+
+	my $str;
+	if( ref $$obj eq 'Netscape::Bookmarks::Category' )
 		{
-		$str .= TAB x ($level) . START_LIST_ITEM . $obj->as_headline;
-		$str .= TAB x ($level-1) . START_LIST . "\n";
-		
 		++$level;
-		foreach my $ref ( $obj->elements )
+		$str .= TAB x ($level - 1) . START_LIST_ITEM . ($$obj)->as_headline . "\n";
+		$str .= TAB x ($level - 1) . START_LIST . "\n";
+
+		foreach my $ref ( @{($$obj)->elements} )
 			{
-			$str .= $self->_as_string( $ref, $level );
+			$str .= $self->_as_string(\$ref, $level);
 			}
+
+		$str .= TAB x ($level - 1) . END_LIST . "\n";
 		--$level;
-	
-		$str .= TAB x ($level) . END_LIST . "\n";
 		}
-	elsif( ref $obj eq 'Netscape::Bookmarks::Link' or
-		   ref $obj eq 'Netscape::Bookmarks::Alias' )
+	elsif( ref $$obj eq 'Netscape::Bookmarks::Link' )
 		{
-		$str .= TAB x ($level) . START_LIST_ITEM 
-			. $obj->as_string . "\n"
+		my $title = ($$obj)->title;
+		my $url   = ($$obj)->href;
+		$str .= TAB x ($level) . START_LIST_ITEM . ($$obj)->as_string . "\n"
 		}
-	elsif( ref $obj eq 'Netscape::Bookmarks::Separator' )
+	elsif( ref $$obj eq 'Netscape::Bookmarks::Alias' )
 		{
-		$str .= TAB x ($level) . $obj->as_string . "\n"
+		my $title = ($$obj)->target->title;
+		my $url   = ($$obj)->target->href;
+		my $s = ($$obj)->target->as_string;
+		$s =~ s/ALIASID/ALIASOF/;
+		$str .= TAB x ($level) . START_LIST_ITEM . $s . "\n"
 		}
-		
+	elsif( ref $$obj eq 'Netscape::Bookmarks::Separator' )
+		{
+		$str .= TAB x ($level) . ($$obj)->as_string . "\n"
+		}
+
 	return $str;
-		
-	}	
-	
+
+	}
+
 "if you want to beleive everything you read, so be it.";
 
 __END__
 
 =back
 
+=head1 SOURCE AVAILABILITY
+
+This source is part of a SourceForge project which always has the
+latest sources in CVS, as well as all of the previous releases.
+
+	http://sourceforge.net/projects/nsbookmarks/
+
+If, for some reason, I disappear from the world, one of the other
+members of the project can shepherd this module appropriately.
+
 =head1 AUTHOR
 
-brian d foy E<lt>bdfoy@cpan.orgE<gt>
+brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT
 
